/*
 *  Authors:  tiandongdong <tiandongdong@kylinos.cn>
 *  Authors:  litianhao <litianhao@kylinos.cn>
 *
 *  Copyright (c) KylinSoft Co., Ltd [2024].All rights reserved.
 */
#pragma once

#include <stddef.h>

#include "./scene_common.h"

#ifdef __cplusplus
extern "C" {
#endif

struct kysec_scene_filectl_exception_data {
    /* 被保护资源 */
    char *p_resource;
    /* 放行的例外程序 */
    char **e_resource;
    /* 例外程序个数 */
    size_t count;
};

/**
 * @brief 根据路径添加文件保护
 * @param path 文件绝对路径
 * @return 0:成功 -6: 代表只读目录 其他:失败
 */
int kysec_scene_filectl_add_by_path(const char *path);

/**
 * @brief 根据路径删除文件保护
 * @param path 文件绝对路径
 * @return 0:成功 -6: 代表只读目录 其他:失败
 *
 */
int kysec_scene_filectl_remove_by_path(const char *path);

/**
 * @brief 根据路径批量添加文件保护
 * @param paths 文件路径数组
 * @param count 个数
 * @return 0:成功 -6: 代表只读目录 其他:失败
 */
int kysec_scene_filectl_multi_add_by_path(char **paths, size_t count);

/**
 * @brief 根据路径批量删除文件保护
 * @param paths 文件路径数组
 * @param count 个数
 * @return 0:成功 -6: 代表只读目录 其他:失败
 */
int kysec_scene_filectl_multi_remove_by_path(char **paths, size_t count);

/**
 * @brief 获取下发的文件保护策略
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */
char **kysec_scene_filectl_get_policy(size_t *count);

/**
 * @brief 根据软件包添加文件保护
 * (软件包维度,只有包里的程序对包文件有权限,其他程序无权限)
 * @param pkg 软件包名称
 * @return 0:成功 其他:失败
 */
int kysec_scene_filectl_add_by_pkg(const char *pkg);

/**
 * @brief 根据软包删除文件保护
 * @param pkg 软件包名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_filectl_remove_by_pkg(const char *pkg);

/**
 * @brief 获取下发的文件保护策略(软件包维度)
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */

char **kysec_scene_filectl_get_pkg_policy(size_t *count);

/**
 * @brief 根据软件包添加文件保护
 * (软件包维度,只有包里的程序对包文件有权限,其他程序无权限)
 * @param pkg 软件包名称
 * @return 0:成功 其他:失败
 */
int kysec_scene_filectl_multi_add_by_pkg(char *pkg[], size_t count);

/**
 * @brief 根据软包删除文件保护
 * @param pkg 软件包名称
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_filectl_multi_remove_by_pkg(char *pkg[], size_t count);

/**
 * @brief 文件防护例外程序添加
 * @param p_type 保护资源类型  文件/目录/软件包
 * @param p_resource 文件路径/目录路径/软件包名称
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource 例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_filectl_exception_add(enum kysec_scene_resource_type p_type,
                                      char *p_resource,
                                      enum kysec_scene_resource_type e_type,
                                      char *e_resource);

/**
 * @brief 文件防护例外程序删除
 * @param p_type 保护资源类型  文件/目录/软件包
 * @param p_resource 文件路径/目录路径/软件包名称
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource  例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_filectl_exception_remove(enum kysec_scene_resource_type p_type,
                                         char *p_resource,
                                         enum kysec_scene_resource_type e_type,
                                         char *e_resource);

/**
 * @brief 文件防护例外程序批量添加
 * @param p_type 保护资源类型  文件/目录/软件包
 * @param p_resource 文件路径/目录路径/软件包名称
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource  例外程序数组
 * @param count 例外程序个数
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_filectl_exception_multi_add(
    enum kysec_scene_resource_type p_type, char *p_resource,
    enum kysec_scene_resource_type e_type, char *e_resource[], size_t count);

/**
 * @brief 文件防护例外程序批量删除
 * @param p_type 保护资源类型  文件/目录/软件包
 * @param p_resource 文件路径/目录路径/软件包名称
 * @param e_type 例外资源类型   文件/目录/软件包
 * @param e_resource
 * @param count 例外程序个数
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_filectl_exception_multi_remove(
    enum kysec_scene_resource_type p_type, char *p_resource,
    enum kysec_scene_resource_type e_type, char *e_resource[], size_t count);

/**
 * @brief 获取文件防护 例外程序策略
 * @param count 个数
 * @return 成功: 数据数组 失败或没有数据:NULL
 * 使用完需要调用kysec_scene_filectl_exception_policy_free释放内存
 *
 */
struct kysec_scene_filectl_exception_data *
kysec_scene_filectl_exception_get_policy(size_t *count);

/**
 * @brief 释放例外程序策略内存
 * @param ptr 数据指针 (通过kysec_scene_filectl_exception_get_policy 获取的数据)
 * @param count 个数
 * @return 成功: 数据数组 失败或没有数据:NULL
 *
 */
void kysec_scene_filectl_exception_policy_free(
    struct kysec_scene_filectl_exception_data *ptr, size_t count);

#ifdef __cplusplus
}
#endif
