/*
 *  Authors:  tiandongdong <tiandongdong@kylinos.cn>
 *  Authors:  litianhao <litianhao@kylinos.cn>
 *
 *  Copyright (c) KylinSoft Co., Ltd [2024].All rights reserved.
 */
#pragma once

#include <sys/types.h>

#include "./scene_common.h"

#ifdef __cplusplus
extern "C" {
#endif

struct kysec_scene_taskctl_exception_data {
    /* 被保护资源 */
    char *p_resource;
    /* 放行的例外程序 */
    char **e_resource;
    /* 例外程序个数 */
    size_t count;
};

/**
 * @brief 根据路径添加进程防杀死
 * @param path 文件绝对路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_taskctl_add_by_path(const char *path);

/**
 * @brief 根据路径删除进程防杀死
 * @param path 文件绝对路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_taskctl_remove_by_path(const char *path);

/**
 * @brief 根据路径批量添加进程防杀死
 * @param path 文件绝对路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_taskctl_multi_add_by_path(char *paths[], size_t count);

/**
 * @brief 根据路径批量删除进程防杀死
 * @param path 文件绝对路径
 * @return 0:成功 其他:失败
 */
int kysec_scene_taskctl_multi_remove_by_path(char *paths[], size_t count);

/**
 * @brief 根据进程id添加进程防杀死
 * @param pid  进程pid
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_add_by_pid(pid_t pid);

/**
 * @brief 根据进程id删除进程防杀死
 * @param pid  进程pid
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_remove_by_pid(pid_t pid);

/**
 * @brief 根据进程id批量添加进程防杀死
 * @param pid  进程pid数组
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_multi_add_by_pid(pid_t pids[], size_t count);

/**
 * @brief 根据进程id批量删除进程防杀死
 * @param pids 进程pid数组
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_multi_remove_by_pid(pid_t pids[], size_t count);

/**
 * @brief 获取下发的进程保护策略
 * @param count 个数
 * @return 字符串数组  NULL:无数据
 *         调用者使用完需要手动释放内存
 */
char **kysec_scene_taskctl_get_policy(size_t *count);

/**
 * @brief 进程防护例外程序添加
 * @param p_type 保护资源类型  文件
 * @param p_resource 文件路径
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource 例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_add(enum kysec_scene_resource_type p_type,
                                      char *p_resource,
                                      enum kysec_scene_resource_type e_type,
                                      char *e_resource);

/**
 * @brief 进程防护例外程序删除
 * @param p_type 保护资源类型  文件 (只能是文件路径)
 * @param p_resource 文件路径 (只能是文件路径)
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource  例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_remove(enum kysec_scene_resource_type p_type,
                                         char *p_resource,
                                         enum kysec_scene_resource_type e_type,
                                         char *e_resource);

/**
 * @brief 进程防护例外程序批量添加
 * @param p_type 保护资源类型  文件 (只能是文件路径)
 * @param p_resource 文件路径 (只能是文件路径)
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource  例外程序数组
 * @param count 例外程序个数
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_multi_add(
    enum kysec_scene_resource_type p_type, char *p_resource,
    enum kysec_scene_resource_type e_type, char *e_resource[], size_t count);

/**
 * @brief 进程防护例外程序批量删除
 * @param p_type 保护资源类型   文件 (只能是文件路径)
 * @param p_resource 文件路径  文件 (只能是文件路径)
 * @param e_type 例外资源类型   文件/目录/软件包
 * @param e_resource
 * @param count 例外程序个数
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_multi_remove(
    enum kysec_scene_resource_type p_type, char *p_resource,
    enum kysec_scene_resource_type e_type, char *e_resource[], size_t count);

/**
 * @brief 进程防护例外程序添加 (pid)
 * @param p_pid 保护对象(pid)
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource 例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_add_by_pid(
    pid_t p_pid, enum kysec_scene_resource_type e_type, char *e_resource);

/**
 * @brief 进程防护例外程序删除 (pid)
 * @param p_pid 保护对象(pid)
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource 例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_remove_by_pid(
    pid_t pid, enum kysec_scene_resource_type e_type, char *e_resource);

/**
 * @brief 进程防护例外程序批量添加
 * @param p_pid 保护对象(pid)
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource 例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_multi_add_by_pid(
    pid_t pid, enum kysec_scene_resource_type e_type, char *e_resource[],
    size_t count);

/**
 * @brief 进程防护例外程序批量删除
 * @param p_pid 保护对象(pid)
 * @param e_type 例外程序类型   文件/目录/软件包
 * @param e_resource 例外程序
 * @return 0:成功 其他:失败
 *
 */
int kysec_scene_taskctl_exception_multi_remove_by_pid(
    pid_t pid, enum kysec_scene_resource_type e_type, char *e_resource[],
    size_t count);

/**
 * @brief 获取文件防护 例外程序策略
 * @param count 个数
 * @return 成功: 数据数组 失败或没有数据:NULL
 * 使用完需要调用kysec_scene_taskctl_exception_policy_free释放内存
 *
 */
struct kysec_scene_taskctl_exception_data *
kysec_scene_taskctl_exception_get_policy(size_t *count);

/**
 * @brief 释放例外程序策略内存
 * @param ptr 数据指针 (通过kysec_scene_taskctl_exception_get_policy 获取的数据)
 * @param count 个数
 * @return 成功: 数据数组 失败或没有数据:NULL
 *
 */
void kysec_scene_taskctl_exception_policy_free(
    struct kysec_scene_taskctl_exception_data *ptr, size_t count);
#ifdef __cplusplus
}
#endif
